import { CommandSet, Action, ActionType, Command } from "src/model/command";
import { Message, TextChannel } from "discord.js";
import { getRepository } from "typeorm";
import { FileOnlyChannel } from "@shared/db/entity/FileOnlyChannel";

@CommandSet
export class FileOnlyChannelChecker {
    private urlPattern: RegExp = new RegExp("(?:((?:https?|ftp):\/\/)|ww)(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,}))\.?)(?::\d{2,5})?(?:[/?#]\S*)?");

    @Action(ActionType.MESSAGE)
    async processMessage(actionsDone: boolean, msg: Message, content: string) {
        if (actionsDone)
            return false;

        let repo = getRepository(FileOnlyChannel);
        let entry = await repo.findOne(msg.channel.id);
        if (!entry)
            return false;

        // Has attachments; is fine
        if (msg.attachments.size > 0)
            return false;

        // Has a link
        if(this.urlPattern.test(msg.content))
            return false;

        msg.delete();

        let ch = msg.guild.channels.resolve(entry.warningMessageChannelId);
        
        if(ch instanceof TextChannel)
            ch.send(`> ${msg.content.replace(/\n/g, "\n> ")}\n\n${msg.author.toString()} Channel ${msg.channel.toString()} is only meant for sharing links and files! If you want to post text-only messages, do so in ${ch.toString()}!`);

        return true;
    }
};